"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const util = require("util");
const file_1 = require("./file");
class StackOutputPlugin {
    constructor(serverless, options) {
        this.serverless = serverless;
        this.options = options;
        this.hooks = {
            'after:deploy:deploy': this.process.bind(this)
        };
        this.output = this.serverless.service.custom.output;
    }
    get file() {
        return this.getConfig('file');
    }
    get handler() {
        return this.getConfig('handler');
    }
    get stackName() {
        return this.serverless.service.getServiceName() + '-' + this.serverless.getProvider('aws').getStage();
    }
    hasConfig(key) {
        return !!this.output && !!this.output[key];
    }
    hasHandler() {
        return this.hasConfig('handler');
    }
    hasFile() {
        return this.hasConfig('file');
    }
    getConfig(key) {
        return this.serverless.config.servicePath + '/' + this.output[key];
    }
    callHandler(data) {
        const splits = this.handler.split('.');
        const func = splits.pop() || '';
        return new Promise((resolve) => {
            require(splits.join('.'))[func](data);
            resolve();
        });
    }
    saveFile(data) {
        const f = new file_1.default(this.file);
        return new Promise((resolve) => {
            f.save(data);
            resolve();
        });
    }
    fetch() {
        return this.serverless.getProvider('aws').request('CloudFormation', 'describeStacks', {
            StackName: this.stackName
        }, this.serverless.getProvider('aws').getStage(), this.serverless.getProvider('aws').getRegion());
    }
    beautify(data) {
        const stack = data.Stacks.pop() || { Outputs: [] };
        const output = stack.Outputs || [];
        return output.reduce((obj, item) => Object.assign(obj, { [item.OutputKey]: item.OutputValue }), {});
    }
    handle(data) {
        const promises = [];
        if (this.hasHandler()) {
            promises.push(this.callHandler(data).then(() => this.serverless.cli.log(util.format('Stack Output processed with handler: %s', this.output.handler))));
        }
        if (this.hasFile()) {
            promises.push(this.saveFile(data).then(() => this.serverless.cli.log(util.format('Stack Output saved to file: %s', this.output.file))));
        }
        return Promise.all(promises);
    }
    validate() {
        assert(this.serverless, 'Invalid serverless configuration');
        assert(this.serverless.service, 'Invalid serverless configuration');
        assert(this.serverless.service.provider, 'Invalid serverless configuration');
        assert(this.serverless.service.provider.name, 'Invalid serverless configuration');
        assert(this.serverless.service.provider.name === 'aws', 'Only supported for AWS provider');
        assert(this.options && !this.options.noDeploy, 'Skipping deployment with --noDeploy flag');
    }
    process() {
        console.log('running stack-output-plugin');
        return Promise.resolve().then(() => this.validate()).then(() => this.fetch()).then((res) => this.beautify(res)).then((res) => this.handle(res)).catch((err) => this.serverless.cli.log(util.format('Cannot process Stack Output: %s!', err.message)));
    }
}
module.exports = StackOutputPlugin;
//# sourceMappingURL=plugin.js.map